# encoding: UTF-8
#
# Copyright (c) 2015 Michel Martens
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

class Tas
  attr_accessor :params
  attr_accessor :reduce

  MarkerMissing = Class.new(ArgumentError)
  ReduceMissing = Class.new(ArgumentError)

  DEFAULT = -> (params) {
    raise(ReduceMissing)
  }

  def initialize(params = {}, &reduce)
    @params = params
    @reduce = reduce || DEFAULT
  end

  def fetch(marker)
    @params.fetch(marker) do
      raise(MarkerMissing, marker)
    end
  end

  def [](marker)
    @params[marker]
  end

  def []=(marker, component)
    @params[marker] = component
  end

  def update(params)
    @params.update(params)
  end

  def to_s
    @reduce[@params]
  end

  def new
    self.class.new(&reduce)
  end

  alias inspect to_s
end
